/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.fitting.excel;


import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 服务商向平台备件申请领用记录表 Excel实体类
 *
 * @author lzp
 * @since 2024-08-24
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class PlatformFittingApplyExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * 申请编号，生成规则：yyMMddHHmmssSSS
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请编号，生成规则：yyMMddHHmmssSSS")
	private String number;
	/**
	 * 运维商id
	 */
	@ColumnWidth(20)
	@ExcelProperty("运维商id")
	private String siteId;
	/**
	 * 运维商名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("运维商名称")
	private String siteName;
	/**
	 * 申请人id
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请人id")
	private String applicantId;
	/**
	 * 申请人姓名
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请人姓名")
	private String applicantName;
	/**
	 * 申请人角色类型：0服务商 
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请人角色类型：0服务商 ")
	private String applicantType;
	/**
	 * 申请备注
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请备注")
	private String applyMarks;
	/**
	 * 审核备注
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核备注")
	private String factoryMarks;
	/**
	 * 出库备注
	 */
	@ColumnWidth(20)
	@ExcelProperty("出库备注")
	private String outFittingMarks;
	/**
	 * 出库确认人
	 */
	@ColumnWidth(20)
	@ExcelProperty("出库确认人")
	private String confirmor;
	/**
	 * 出库确认人 user_id
	 */
	@ColumnWidth(20)
	@ExcelProperty("出库确认人 user_id")
	private String confirmorId;
	/**
	 * 旧件返还标记：0无需返还 1需要返还，默认0
	 */
	@ColumnWidth(20)
	@ExcelProperty("旧件返还标记：0无需返还 1需要返还，默认0")
	private String oldFittingFlag;
	/**
	 * 申请备件id
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请备件id")
	private String fittingId;
	/**
	 * 备件条码
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件条码")
	private String fittingCode;
	/**
	 * 备件型号
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件型号")
	private String fittingVersion;
	/**
	 * 备件名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件名称")
	private String fittingName;
	/**
	 * 申请的备件描述图片地址
	 */
	@ColumnWidth(20)
	@ExcelProperty("申请的备件描述图片地址")
	private String fittingImg;
	/**
	 * 备件申请数目
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件申请数目")
	private BigDecimal applyNum;
	/**
	 * 备件审核数量
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件审核数量")
	private BigDecimal fittingAuditNum;
	/**
	 * 审核时间
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核时间")
	private Date confirmTime;
	/**
	 * 出库时间
	 */
	@ColumnWidth(20)
	@ExcelProperty("出库时间")
	private Date outFittingTime;
	/**
	 * 确认收货时间（入库时间）
	 */
	@ColumnWidth(20)
	@ExcelProperty("确认收货时间（入库时间）")
	private Date sendTime;
	/**
	 * 确认入库人id
	 */
	@ColumnWidth(20)
	@ExcelProperty("确认入库人id")
	private String confirmStorageId;
	/**
	 * 确认入库人
	 */
	@ColumnWidth(20)
	@ExcelProperty("确认入库人")
	private String confirmStorage;
	/**
	 * 物流名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("物流名称")
	private String logisticsNames;
	/**
	 * 物流单号
	 */
	@ColumnWidth(20)
	@ExcelProperty("物流单号")
	private String logisticsNumber;
	/**
	 * 备件签收时间/取消时间/删除时间
	 */
	@ColumnWidth(20)
	@ExcelProperty("备件签收时间/取消时间/删除时间")
	private Date endTime;
	/**
	 * 创建人姓名
	 */
	@ColumnWidth(20)
	@ExcelProperty("创建人姓名")
	private String creator;
	/**
	 * 更新人
	 */
	@ColumnWidth(20)
	@ExcelProperty("更新人")
	private String updateName;
	/**
	 * 收货人姓名
	 */
	@ColumnWidth(20)
	@ExcelProperty("收货人姓名")
	private String deliveryName;
	/**
	 * 收货联系方式
	 */
	@ColumnWidth(20)
	@ExcelProperty("收货联系方式")
	private String deliveryMobile;
	/**
	 * 收货详细地址
	 */
	@ColumnWidth(20)
	@ExcelProperty("收货详细地址")
	private String deliveryAddress;
	/**
	 * 拒绝理由
	 */
	@ColumnWidth(20)
	@ExcelProperty("拒绝理由")
	private String refuseReason;
	/**
	 * 审核人
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核人")
	private String auditor;
	/**
	 * 审核人   user_id
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核人   user_id")
	private String auditorId;
	/**
	 * 适用机型
	 */
	@ColumnWidth(20)
	@ExcelProperty("适用机型")
	private String suitPrinter;
	/**
	 * 服务商价格
	 */
	@ColumnWidth(20)
	@ExcelProperty("服务商价格")
	private BigDecimal sitePrice;
	/**
	 * 零售价格（用户价格）
	 */
	@ColumnWidth(20)
	@ExcelProperty("零售价格（用户价格）")
	private BigDecimal customerPrice;

}
