/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.flowable.examineInfo.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.flowable.examineInfo.pojo.entity.ExamineInfoEntity;
import org.springblade.modules.flowable.examineInfo.pojo.vo.ExamineInfoVO;
import org.springblade.modules.flowable.examineInfo.wrapper.ExamineInfoWrapper;
import org.springblade.modules.flowable.examineInfo.service.IExamineInfoService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 审批明细记录表 控制器
 *
 * @author lzp
 * @since 2024-08-09
 */
@RestController
@AllArgsConstructor
@RequestMapping("blade-examineInfo/examineInfo")
@Tag(name = "审批明细记录表", description = "审批明细记录表接口")
public class ExamineInfoController extends BladeController {

	private final IExamineInfoService examineInfoService;

	/**
	 * 审批明细记录表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入examineInfo")
	public R<ExamineInfoVO> detail(ExamineInfoEntity examineInfo) {
		ExamineInfoEntity detail = examineInfoService.getOne(Condition.getQueryWrapper(examineInfo));
		return R.data(ExamineInfoWrapper.build().entityVO(detail));
	}
	/**
	 * 审批明细记录表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入examineInfo")
	public R<IPage<ExamineInfoVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> examineInfo, Query query) {
		IPage<ExamineInfoEntity> pages = examineInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(examineInfo, ExamineInfoEntity.class));
		return R.data(ExamineInfoWrapper.build().pageVO(pages));
	}

	/**
	 * 审批明细记录表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入examineInfo")
	public R<IPage<ExamineInfoVO>> page(ExamineInfoVO examineInfo, Query query) {
		IPage<ExamineInfoVO> pages = examineInfoService.selectExamineInfoPage(Condition.getPage(query), examineInfo);
		return R.data(pages);
	}

	/**
	 * 审批明细记录表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入examineInfo")
	public R save(@Valid @RequestBody ExamineInfoEntity examineInfo) {
		return R.status(examineInfoService.save(examineInfo));
	}

	/**
	 * 审批明细记录表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入examineInfo")
	public R update(@Valid @RequestBody ExamineInfoEntity examineInfo) {
		return R.status(examineInfoService.updateById(examineInfo));
	}

	/**
	 * 审批明细记录表 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入examineInfo")
	public R submit(@Valid @RequestBody ExamineInfoEntity examineInfo) {
		return R.status(examineInfoService.saveOrUpdate(examineInfo));
	}



}
