/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.flowable.processForm.excel;


import lombok.Data;

import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 流程中的一个审核步骤（或者称为一个任务）。 Excel实体类
 *
 * @author lzp
 * @since 2024-08-02
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class ProcessDefAuditStepExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * 存当前任务名称（例如：1级审批）
	 */
	@ColumnWidth(20)
	@ExcelProperty("存当前任务名称（例如：1级审批）")
	private String name;
	/**
	 * 可见字段，使用逗号分隔，存code。
	 */
	@ColumnWidth(20)
	@ExcelProperty("可见字段，使用逗号分隔，存code。")
	private String visibleFields;
	/**
	 * 可编辑字段，使用逗号分隔，存code。
	 */
	@ColumnWidth(20)
	@ExcelProperty("可编辑字段，使用逗号分隔，存code。")
	private String operateFields;
	/**
	 * 审批角色(审批人类型是平台角色的时候存角色)
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批角色(审批人类型是平台角色的时候存角色)")
	private String roleId;
	/**
	 * 节点名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("节点名称")
	private String itemName;
	/**
	 * 审批人类型 1平台角色 2 平台员工 
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批人类型 1平台角色 2 平台员工 ")
	private String auditType;
	/**
	 * 角色名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("角色名称")
	private String roleName;
	/**
	 * 可审批员工(审批人类型是平台员工的时候存用户id)
	 */
	@ColumnWidth(20)
	@ExcelProperty("可审批员工(审批人类型是平台员工的时候存用户id)")
	private String empUserId;
	/**
	 * 审批方式 1 或签 2会签  3 依次审批 4 任意人
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批方式 1 或签 2会签  3 依次审批 4 任意人")
	private String auditMethod;
	/**
	 * 驳回至某一个节点的id
	 */
	@ColumnWidth(20)
	@ExcelProperty("驳回至某一个节点的id")
	private String rejectStepId;
	/**
	 * 驳回节点类型（0：提交节点，1：审核节点）
	 */
	@ColumnWidth(20)
	@ExcelProperty("驳回节点类型（0：提交节点，1：审核节点）")
	private String rejectStepType;
	/**
	 * 驳回节点名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("驳回节点名称")
	private String rejectStepName;
	/**
	 * 抄送人员id(多个用逗号隔开)
	 */
	@ColumnWidth(20)
	@ExcelProperty("抄送人员id(多个用逗号隔开)")
	private String ccUserId;
	/**
	 * 审批提示
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批提示")
	private String auditNote;
	/**
	 * 可跨部门(0 否  1是)
	 */
	@ColumnWidth(20)
	@ExcelProperty("可跨部门(0 否  1是)")
	private String transDepartment;
	/**
	 * 流程模板id
	 */
	@ColumnWidth(20)
	@ExcelProperty("流程模板id")
	private String procDefId;
	/**
	 * 创建人
	 */
	@ColumnWidth(20)
	@ExcelProperty("创建人")
	private String createBy;
	/**
	 * 审批顺序（小的排在前面）
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批顺序（小的排在前面）")
	private Integer sort;
	/**
	 * 审批状态
	 */
	@ColumnWidth(20)
	@ExcelProperty("审批状态")
	private String stepName;

}
