/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.material.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.material.pojo.entity.MaterialOutboundEntity;
import org.springblade.modules.material.pojo.vo.EmpStockConditionVo;
import org.springblade.modules.operation.mapper.OperationSiteEmployeeMapper;
import org.springblade.modules.operation.pojo.entity.OperationSiteEntity;
import org.springblade.modules.operation.pojo.vo.EmpAuthVo;
import org.springblade.modules.operation.service.impl.OperationSiteServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.material.pojo.entity.MaterialEngineerEntity;
import org.springblade.modules.material.pojo.vo.MaterialEngineerVO;
import org.springblade.modules.material.excel.MaterialEngineerExcel;
import org.springblade.modules.material.wrapper.MaterialEngineerWrapper;
import org.springblade.modules.material.service.IMaterialEngineerService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;

import java.util.HashMap;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工程师库存 控制器
 *
 * @author cswang
 * @since 2024-08-25
 */
@RestController
@AllArgsConstructor
@RequestMapping("wl/materialEngineer")
@Tag(name = "工程师库存", description = "工程师库存接口")
public class MaterialEngineerController extends BladeController {

	private final IMaterialEngineerService materialEngineerService;
	@Autowired
	OperationSiteServiceImpl siteService;

	@Autowired
	OperationSiteEmployeeMapper employeeMapper;
	/**
	 * 工程师库存 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入materialEngineer")
	public R<MaterialEngineerVO> detail(MaterialEngineerEntity materialEngineer) {
		MaterialEngineerEntity detail = materialEngineerService.getOne(Condition.getQueryWrapper(materialEngineer));
		return R.data(MaterialEngineerWrapper.build().entityVO(detail));
	}
	/**
	 * 工程师库存 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入materialEngineer")
	public R<IPage<MaterialEngineerVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> materialEngineer, Query query) {
		BladeUser user = AuthUtil.getUser();
		//3.服务商 4.平台员工 5.服务商人员
		QueryWrapper<MaterialEngineerEntity> queryWrapper = Condition.getQueryWrapper(materialEngineer, MaterialEngineerEntity.class);
		if ("3".equals(user.getDeptId())) {  //服务商
			OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
			queryWrapper.lambda().eq(MaterialEngineerEntity :: getOperationId,site.getId());

		} else if ("5".equals(user.getDeptId())) {  //服务商人员
			EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
			if(authVo==null){
				return R.data(null);
			}
			queryWrapper.lambda().eq(MaterialEngineerEntity :: getOperationId,authVo.getSiteId());
		}


		IPage<MaterialEngineerEntity> pages = materialEngineerService.page(Condition.getPage(query), queryWrapper);
		return R.data(MaterialEngineerWrapper.build().pageVO(pages));
	}

	/**
	 * 工程师库存 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入materialEngineer")
	public R<IPage<MaterialEngineerVO>> page(MaterialEngineerVO materialEngineer, Query query) {
		IPage<MaterialEngineerVO> pages = materialEngineerService.selectMaterialEngineerPage(Condition.getPage(query), materialEngineer);
		return R.data(pages);
	}

	/**
	 * 工程师库存 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入materialEngineer")
	public R save(@Valid @RequestBody MaterialEngineerEntity materialEngineer) {
		return R.status(materialEngineerService.save(materialEngineer));
	}

	/**
	 * 工程师库存 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入materialEngineer")
	public R update(@Valid @RequestBody MaterialEngineerEntity materialEngineer) {
		return R.status(materialEngineerService.updateById(materialEngineer));
	}

	/**
	 * 工程师库存 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入materialEngineer")
	public R submit(@Valid @RequestBody MaterialEngineerEntity materialEngineer) {
		return R.status(materialEngineerService.saveOrUpdate(materialEngineer));
	}

	/**
	 * 工程师库存 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(materialEngineerService.removeByIds(Func.toLongList(ids)));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-materialEngineer")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入materialEngineer")
	public void exportMaterialEngineer(@Parameter(hidden = true) @RequestParam Map<String, Object> materialEngineer, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<MaterialEngineerEntity> queryWrapper = Condition.getQueryWrapper(materialEngineer, MaterialEngineerEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(MaterialEngineer::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(MaterialEngineerEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<MaterialEngineerExcel> list = materialEngineerService.exportMaterialEngineer(queryWrapper);
		ExcelUtil.export(response, "工程师库存数据" + DateUtil.time(), "工程师库存数据表", list, MaterialEngineerExcel.class);
	}


	/**
	 * 工程师库存
	 */
	@GetMapping("/engineerStockList")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "工程师库存--可用物料")
	public R engineerStockList(@RequestParam Map<String, Object>  params, Query query) {
		Map<String,Object> resultMap=new HashMap<>();
		List<MaterialEngineerVO> list=materialEngineerService.engineerStockList(Condition.getPage(query),params);
		resultMap.put("records",list);
		if(params.get("current")!=null){  //为了pc这边分页用的
         Long count=materialEngineerService.countEngineerStockList(params);
			resultMap.put("total",count);
		}
		return R.data(resultMap);
	}

}
