/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.material.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.material.excel.MaterialOutboundExcel;
import org.springblade.modules.material.pojo.entity.MaterialOutboundEntity;
import org.springblade.modules.material.pojo.vo.MaterialOutboundVO;
import org.springblade.modules.material.service.IMaterialOutboundService;
import org.springblade.modules.material.wrapper.MaterialOutboundWrapper;
import org.springblade.modules.operation.mapper.OperationSiteEmployeeMapper;
import org.springblade.modules.operation.pojo.entity.OperationSiteEntity;
import org.springblade.modules.operation.pojo.vo.EmpAuthVo;
import org.springblade.modules.operation.service.impl.OperationSiteServiceImpl;
import org.springblade.modules.system.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.material.pojo.entity.MaterialEngineerOutboundEntity;
import org.springblade.modules.material.pojo.vo.MaterialEngineerOutboundVO;
import org.springblade.modules.material.excel.MaterialEngineerOutboundExcel;
import org.springblade.modules.material.wrapper.MaterialEngineerOutboundWrapper;
import org.springblade.modules.material.service.IMaterialEngineerOutboundService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工程师库存明细 控制器
 *
 * @author AlexWang
 * @since 2024-09-04
 */
@RestController
@AllArgsConstructor
@RequestMapping("wl/materialEngineerOutbound")
@Tag(name = "工程师库存明细", description = "工程师库存明细接口")
public class MaterialEngineerOutboundController extends BladeController {

	private final IMaterialEngineerOutboundService materialEngineerOutboundService;
	private final IUserService userService;

	private final IMaterialOutboundService materialOutboundService;
	@Autowired
	OperationSiteServiceImpl siteService;

	@Autowired
	OperationSiteEmployeeMapper employeeMapper;
	/**
	 * 工程师库存明细 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入materialEngineerOutbound")
	public R<MaterialEngineerOutboundVO> detail(MaterialEngineerOutboundEntity materialEngineerOutbound) {
		MaterialEngineerOutboundEntity detail = materialEngineerOutboundService.getOne(Condition.getQueryWrapper(materialEngineerOutbound));
		return R.data(MaterialEngineerOutboundWrapper.build().entityVO(detail));
	}
	/**
	 * 工程师库存明细 分页
	 */
/*
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入materialEngineerOutbound")
	public R<IPage<MaterialOutboundVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> materialEngineerOutbound, Query query) {
		BladeUser user = AuthUtil.getUser();
		query.setDescs("id");
		QueryWrapper<MaterialOutboundEntity> queryWrapper = Condition.getQueryWrapper(materialEngineerOutbound, MaterialOutboundEntity.class);
		queryWrapper.lambda().eq(MaterialOutboundEntity :: getKeepType,"1");
		if ("3".equals(user.getDeptId())) {  //服务商
			OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
			queryWrapper.lambda().eq(MaterialOutboundEntity:: getCompanyId,site.getId());

		} else if ("5".equals(user.getDeptId())) {  //服务商人员
			EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
			if(authVo==null){
				return R.data(null);
			}
			queryWrapper.lambda().eq(MaterialOutboundEntity :: getCompanyId,authVo.getSiteId());
		}

		IPage<MaterialOutboundEntity> pages = materialOutboundService.page(Condition.getPage(query),queryWrapper);
		IPage<MaterialOutboundVO> pageVO = MaterialOutboundWrapper.build().pageVO(pages);
		for(MaterialOutboundVO vo : pageVO.getRecords()){
			vo.setCreateName(userService.getById(vo.getCreateUser()).getName());
			if(vo.getAllowUser()!=null){
				vo.setAllowName(userService.getById(vo.getAllowUser()).getName());
			}
		}
		return R.data(pageVO);
	}
*/

	/**
	 * 工程师库存明细 自定义分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入materialEngineerOutbound")
	public R<IPage<MaterialOutboundVO>> page(MaterialOutboundVO materialEngineerOutbound, Query query) {
		BladeUser user = AuthUtil.getUser();

		if ("3".equals(user.getDeptId())) {  //服务商
			OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
			materialEngineerOutbound.setCompanyId(site.getId());

		} else if ("5".equals(user.getDeptId())) {  //服务商人员
			EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
			if(authVo==null){
				return R.data(null);
			}
			materialEngineerOutbound.setCompanyId(authVo.getSiteId());
		}

		IPage<MaterialOutboundVO> pages = materialEngineerOutboundService.findMaterialOutbound(Condition.getPage(query), materialEngineerOutbound);
		return R.data(pages);
	}



	/**
	 * 导出数据
	 */
	@GetMapping("/export-materialEngineerOutbound")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入materialEngineerOutbound")
	public void exportMaterialEngineerOutbound(@Parameter(hidden = true) @RequestParam Map<String, Object> materialEngineerOutbound, BladeUser bladeUser, HttpServletResponse response) {
		BladeUser user = AuthUtil.getUser();
		QueryWrapper<MaterialOutboundEntity> queryWrapper = Condition.getQueryWrapper(materialEngineerOutbound, MaterialOutboundEntity.class);
		queryWrapper.lambda().eq(MaterialOutboundEntity :: getKeepType,"1");
		if ("3".equals(user.getDeptId())) {  //服务商
			OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
			queryWrapper.lambda().eq(MaterialOutboundEntity:: getCompanyId,site.getId());

		} else if ("5".equals(user.getDeptId())) {  //服务商人员
			EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
			if(authVo==null){
				authVo = new EmpAuthVo();
			}
			queryWrapper.lambda().eq(MaterialOutboundEntity :: getCompanyId,authVo.getSiteId());
		}
		queryWrapper.lambda().eq(MaterialOutboundEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<MaterialEngineerOutboundExcel> list = materialEngineerOutboundService.exportMaterialEngineerOutbound(queryWrapper);
		ExcelUtil.export(response, "工程师库存明细数据" + DateUtil.time(), "工程师库存明细数据表", list, MaterialEngineerOutboundExcel.class);
	}


	/*QueryWrapper<MaterialOutboundEntity> queryWrapper = Condition.getQueryWrapper(materialEngineerOutbound, MaterialOutboundEntity.class);
		queryWrapper.lambda().eq(MaterialOutboundEntity :: getKeepType,"1");
		if ("3".equals(user.getDeptId())) {  //服务商
		OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
		queryWrapper.lambda().eq(MaterialOutboundEntity:: getCompanyId,site.getId());

	} else if ("5".equals(user.getDeptId())) {  //服务商人员
		EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
		if(authVo==null){
			authVo = new EmpAuthVo();
		}
		queryWrapper.lambda().eq(MaterialOutboundEntity :: getCompanyId,authVo.getSiteId());
	}
		queryWrapper.lambda().eq(MaterialOutboundEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
	List<MaterialOutboundExcel> list = materialOutboundService.exportMaterialOutbound(queryWrapper);*/


}
