/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.material.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.material.pojo.entity.MaterialEngineerEntity;
import org.springblade.modules.operation.mapper.OperationSiteEmployeeMapper;
import org.springblade.modules.operation.pojo.entity.OperationSiteEntity;
import org.springblade.modules.operation.pojo.vo.EmpAuthVo;
import org.springblade.modules.operation.service.impl.OperationSiteServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.material.pojo.entity.MaterialOperationEntity;
import org.springblade.modules.material.pojo.vo.MaterialOperationVO;
import org.springblade.modules.material.excel.MaterialOperationExcel;
import org.springblade.modules.material.wrapper.MaterialOperationWrapper;
import org.springblade.modules.material.service.IMaterialOperationService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 运维商库存 控制器
 *
 * @author cswang
 * @since 2024-08-25
 */
@RestController
@AllArgsConstructor
@RequestMapping("wl/materialOperation")
@Tag(name = "运维商库存", description = "运维商库存接口")
public class MaterialOperationController extends BladeController {

	private final IMaterialOperationService materialOperationService;
	@Autowired
	OperationSiteServiceImpl siteService;

	@Autowired
	OperationSiteEmployeeMapper employeeMapper;
	/**
	 * 运维商库存 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入materialOperation")
	public R<MaterialOperationVO> detail(MaterialOperationEntity materialOperation) {
		MaterialOperationEntity detail = materialOperationService.getOne(Condition.getQueryWrapper(materialOperation));
		return R.data(MaterialOperationWrapper.build().entityVO(detail));
	}
	/**
	 * 运维商库存 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入materialOperation")
	public R<IPage<MaterialOperationVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> materialOperation, Query query) {

		BladeUser user = AuthUtil.getUser();
		//3.服务商 4.平台员工 5.服务商人员
		QueryWrapper<MaterialOperationEntity> queryWrapper = Condition.getQueryWrapper(materialOperation, MaterialOperationEntity.class);
		if ("3".equals(user.getDeptId())) {  //服务商
			OperationSiteEntity site = siteService.findSiteByUserId(user.getUserId());
			queryWrapper.lambda().eq(MaterialOperationEntity :: getOperationId,site.getId());

		} else if ("5".equals(user.getDeptId())) {  //服务商人员
			EmpAuthVo authVo=employeeMapper.findEmpAuthByUserId(user.getUserId());
			if(authVo==null){
				return R.data(null);
			}
			queryWrapper.lambda().eq(MaterialOperationEntity :: getOperationId,authVo.getSiteId());
		}
		IPage<MaterialOperationEntity> pages = materialOperationService.page(Condition.getPage(query), queryWrapper);
		return R.data(MaterialOperationWrapper.build().pageVO(pages));
	}

	/**
	 * 运维商库存 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入materialOperation")
	public R<IPage<MaterialOperationVO>> page(MaterialOperationVO materialOperation, Query query) {
		IPage<MaterialOperationVO> pages = materialOperationService.selectMaterialOperationPage(Condition.getPage(query), materialOperation);
		return R.data(pages);
	}

	/**
	 * 运维商库存 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入materialOperation")
	public R save(@Valid @RequestBody MaterialOperationEntity materialOperation) {
		return R.status(materialOperationService.save(materialOperation));
	}

	/**
	 * 运维商库存 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入materialOperation")
	public R update(@Valid @RequestBody MaterialOperationEntity materialOperation) {
		return R.status(materialOperationService.updateById(materialOperation));
	}

	/**
	 * 运维商库存 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入materialOperation")
	public R submit(@Valid @RequestBody MaterialOperationEntity materialOperation) {
		return R.status(materialOperationService.saveOrUpdate(materialOperation));
	}

	/**
	 * 运维商库存 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(materialOperationService.removeByIds(Func.toLongList(ids)));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-materialOperation")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入materialOperation")
	public void exportMaterialOperation(@Parameter(hidden = true) @RequestParam Map<String, Object> materialOperation, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<MaterialOperationEntity> queryWrapper = Condition.getQueryWrapper(materialOperation, MaterialOperationEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(MaterialOperation::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(MaterialOperationEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<MaterialOperationExcel> list = materialOperationService.exportMaterialOperation(queryWrapper);
		ExcelUtil.export(response, "运维商库存数据" + DateUtil.time(), "运维商库存数据表", list, MaterialOperationExcel.class);
	}

}
