/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.nh.pojo.entity.MonitorEntryMonthStatEntity;
import org.springblade.modules.nh.pojo.vo.MonitorEntryMonthStatVO;
import org.springblade.modules.nh.excel.MonitorEntryMonthStatExcel;
import org.springblade.modules.nh.wrapper.MonitorEntryMonthStatWrapper;
import org.springblade.modules.nh.service.IMonitorEntryMonthStatService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 电站月统计数据 控制器
 *
 * @author AlexWang
 * @since 2024-06-02
 */
@RestController
@AllArgsConstructor
@RequestMapping("nh/monitorEntryMonthStat")
@Tag(name = "电站月统计数据", description = "电站月统计数据接口")
public class MonitorEntryMonthStatController extends BladeController {

	private final IMonitorEntryMonthStatService monitorEntryMonthStatService;

	/**
	 * 电站月统计数据 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入monitorEntryMonthStat")
	public R<MonitorEntryMonthStatVO> detail(MonitorEntryMonthStatEntity monitorEntryMonthStat) {
		MonitorEntryMonthStatEntity detail = monitorEntryMonthStatService.getOne(Condition.getQueryWrapper(monitorEntryMonthStat));
		return R.data(MonitorEntryMonthStatWrapper.build().entityVO(detail));
	}
	/**
	 * 电站月统计数据 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入monitorEntryMonthStat")
	public R<IPage<MonitorEntryMonthStatVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> monitorEntryMonthStat, Query query) {
		IPage<MonitorEntryMonthStatEntity> pages = monitorEntryMonthStatService.page(Condition.getPage(query), Condition.getQueryWrapper(monitorEntryMonthStat, MonitorEntryMonthStatEntity.class));
		return R.data(MonitorEntryMonthStatWrapper.build().pageVO(pages));
	}

	/**
	 * 电站月统计数据 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入monitorEntryMonthStat")
	public R<IPage<MonitorEntryMonthStatVO>> page(MonitorEntryMonthStatVO monitorEntryMonthStat, Query query) {
		IPage<MonitorEntryMonthStatVO> pages = monitorEntryMonthStatService.selectMonitorEntryMonthStatPage(Condition.getPage(query), monitorEntryMonthStat);
		return R.data(pages);
	}

	/**
	 * 电站月统计数据 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入monitorEntryMonthStat")
	public R save(@Valid @RequestBody MonitorEntryMonthStatEntity monitorEntryMonthStat) {
		return R.status(monitorEntryMonthStatService.save(monitorEntryMonthStat));
	}

	/**
	 * 电站月统计数据 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入monitorEntryMonthStat")
	public R update(@Valid @RequestBody MonitorEntryMonthStatEntity monitorEntryMonthStat) {
		return R.status(monitorEntryMonthStatService.updateById(monitorEntryMonthStat));
	}

	/**
	 * 电站月统计数据 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入monitorEntryMonthStat")
	public R submit(@Valid @RequestBody MonitorEntryMonthStatEntity monitorEntryMonthStat) {
		return R.status(monitorEntryMonthStatService.saveOrUpdate(monitorEntryMonthStat));
	}

	/**
	 * 电站月统计数据 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(monitorEntryMonthStatService.removeByIds(Func.toLongList(ids)));
	}



}
