/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.operation.pojo.dto.OperationServiceMeasuresDTO;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationServiceMeasuresEntity;
import org.springblade.modules.operation.pojo.vo.OperationServiceMeasuresVO;
import org.springblade.modules.operation.excel.OperationServiceMeasuresExcel;
import org.springblade.modules.operation.wrapper.OperationServiceMeasuresWrapper;
import org.springblade.modules.operation.service.IOperationServiceMeasuresService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 服务措施表 控制器
 *
 * @author lzp
 * @since 2024-06-18
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationServiceMeasures")
@Tag(name = "服务措施表", description = "服务措施表接口")
public class OperationServiceMeasuresController extends BladeController {

	private final IOperationServiceMeasuresService operationServiceMeasuresService;

	/**
	 * 服务措施表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationServiceMeasures")
	public R<OperationServiceMeasuresVO> detail(OperationServiceMeasuresEntity operationServiceMeasures) {
		OperationServiceMeasuresEntity detail = operationServiceMeasuresService.getOne(Condition.getQueryWrapper(operationServiceMeasures));
		return R.data(OperationServiceMeasuresWrapper.build().entityVO(detail));
	}
	/**
	 * 服务措施表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入operationServiceMeasures")
	public R<IPage<OperationServiceMeasuresVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> operationServiceMeasures, Query query) {
		IPage<OperationServiceMeasuresEntity> pages = operationServiceMeasuresService.page(Condition.getPage(query), Condition.getQueryWrapper(operationServiceMeasures, OperationServiceMeasuresEntity.class));
		return R.data(OperationServiceMeasuresWrapper.build().pageVO(pages));
	}

	/**
	 * 服务措施表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationServiceMeasures")
	public R<IPage<OperationServiceMeasuresVO>> page(OperationServiceMeasuresVO operationServiceMeasures, Query query) {
		IPage<OperationServiceMeasuresVO> pages = operationServiceMeasuresService.selectOperationServiceMeasuresPage(Condition.getPage(query), operationServiceMeasures);
		return R.data(pages);
	}

	/**
	 * 服务措施表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationServiceMeasures")
	public R save(@Valid @RequestBody OperationServiceMeasuresDTO operationServiceMeasures) {
		return R.status(operationServiceMeasuresService.saveServiceMeasures(operationServiceMeasures));
	}

	/**
	 * 服务措施表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationServiceMeasures")
	public R update(@Valid @RequestBody OperationServiceMeasuresEntity operationServiceMeasures) {
		return R.status(operationServiceMeasuresService.updateById(operationServiceMeasures));
	}

	/**
	 * 服务措施表 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入operationServiceMeasures")
	public R submit(@Valid @RequestBody OperationServiceMeasuresEntity operationServiceMeasures) {
		return R.status(operationServiceMeasuresService.saveOrUpdate(operationServiceMeasures));
	}

	/**
	 * 服务措施表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String id) {
		return R.status(operationServiceMeasuresService.removeById(id));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationServiceMeasures")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationServiceMeasures")
	public void exportOperationServiceMeasures(@Parameter(hidden = true) @RequestParam Map<String, Object> operationServiceMeasures, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationServiceMeasuresEntity> queryWrapper = Condition.getQueryWrapper(operationServiceMeasures, OperationServiceMeasuresEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(OperationServiceMeasures::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(OperationServiceMeasuresEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<OperationServiceMeasuresExcel> list = operationServiceMeasuresService.exportOperationServiceMeasures(queryWrapper);
		ExcelUtil.export(response, "服务措施表数据" + DateUtil.time(), "服务措施表数据表", list, OperationServiceMeasuresExcel.class);
	}

}
