/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.operation.pojo.dto.OperationServiceTypeDTO;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationServiceTypeEntity;
import org.springblade.modules.operation.pojo.vo.OperationServiceTypeVO;
import org.springblade.modules.operation.excel.OperationServiceTypeExcel;
import org.springblade.modules.operation.wrapper.OperationServiceTypeWrapper;
import org.springblade.modules.operation.service.IOperationServiceTypeService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 服务类型基础信息 控制器
 *
 * @author BladeX
 * @since 2024-06-17
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationServiceType")
@Tag(name = "服务类型基础信息", description = "服务类型基础信息接口")
public class OperationServiceTypeController extends BladeController {

	private final IOperationServiceTypeService operationServiceTypeService;

	/**
	 * 服务类型基础信息 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationServiceType")
	public R<OperationServiceTypeVO> detail(OperationServiceTypeEntity operationServiceType) {
		OperationServiceTypeEntity detail = operationServiceTypeService.getOne(Condition.getQueryWrapper(operationServiceType));
		return R.data(OperationServiceTypeWrapper.build().entityVO(detail));
	}
	/**
	 * 服务类型基础信息 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "不分页", description  = "传入operationServiceType")
	public R<List<OperationServiceTypeVO>> list() {
		List<OperationServiceTypeVO> pages = operationServiceTypeService.pageServiceType();
		return R.data(pages);
	}

	/**
	 * 服务类型基础信息 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationServiceType")
	public R<IPage<OperationServiceTypeVO>> page(OperationServiceTypeVO operationServiceType, Query query) {
		IPage<OperationServiceTypeVO> pages = operationServiceTypeService.selectOperationServiceTypePage(Condition.getPage(query), operationServiceType);
		return R.data(pages);
	}

	/**
	 * 服务类型基础信息 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationServiceType")
	public R save(@Valid @RequestBody OperationServiceTypeDTO dto) {
		return R.status(operationServiceTypeService.saveList(dto));
	}

	/**
	 * 服务类型基础信息 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationServiceType")
	public R update(@Valid @RequestBody OperationServiceTypeEntity operationServiceType) {
		return R.status(operationServiceTypeService.updateById(operationServiceType));
	}

	/**
	 * 服务类型基础信息 新增或修改
	 */
/*
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入operationServiceType")
	public R submit(@Valid @RequestBody OperationServiceTypeEntity operationServiceType) {
		return R.status(operationServiceTypeService.saveOrUpdate(operationServiceType));
	}
*/

	/**
	 * 服务类型基础信息 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String id) {
		return R.status(operationServiceTypeService.deleteStatus(id));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationServiceType")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationServiceType")
	public void exportOperationServiceType(@Parameter(hidden = true) @RequestParam Map<String, Object> operationServiceType, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationServiceTypeEntity> queryWrapper = Condition.getQueryWrapper(operationServiceType, OperationServiceTypeEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(OperationServiceType::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(OperationServiceTypeEntity::getStatus, BladeConstant.DB_NOT_DELETED);
		List<OperationServiceTypeExcel> list = operationServiceTypeService.exportOperationServiceType(queryWrapper);
		ExcelUtil.export(response, "服务类型基础信息数据" + DateUtil.time(), "服务类型基础信息数据表", list, OperationServiceTypeExcel.class);
	}

}
