/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import org.apache.commons.compress.utils.Lists;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.modules.operation.excel.OperationSiteExcel;
import org.springblade.modules.operation.wrapper.OperationSiteWrapper;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationSiteEntity;
import org.springblade.modules.operation.pojo.vo.OperationSiteVO;
import org.springblade.modules.operation.excel.EntrySiteExcel;
import org.springblade.modules.operation.service.IOperationSiteService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;

import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 网点基础信息表 控制器
 *
 * @author lzp
 * @since 2024-06-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationSite")
@Tag(name = "网点基础信息表", description = "网点基础信息表接口")
public class OperationSiteController extends BladeController {

	private final IOperationSiteService operationSiteService;

	/**
	 * 网点基础信息表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationSite")
	public R<OperationSiteVO> detail(@Parameter(name = "主键id", required = true) @RequestParam String id) {
		OperationSiteVO detail = operationSiteService.getSiteDetail(id);
		return R.data(detail);
	}


	/**
	 * 网点基础信息表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "不分页", description  = "传入operationSite")
	public R<List<OperationSiteEntity>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> operationSite) {
		operationSite.put("status","0");
		List<OperationSiteEntity> list = operationSiteService.list(Condition.getQueryWrapper(operationSite, OperationSiteEntity.class));
		return R.data(list);
	}

	/**
	 * 网点基础信息表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationSite")
	public R<IPage<OperationSiteVO>> page(OperationSiteVO operationSite, Query query) {
		IPage<OperationSiteVO> pages = operationSiteService.selectOperationSitePage(Condition.getPage(query), operationSite);
		return R.data(pages);
	}


	/**
	 * 网点基础信息表 自定义分页
	 */
	@GetMapping("/findSiteAmountPage")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "结算账户", description  = "传入operationSite")
	public R<IPage<OperationSiteVO>> findSiteAmountPage(OperationSiteVO operationSite, Query query) {
		IPage<OperationSiteVO> pages = operationSiteService.findSiteAmountPage(Condition.getPage(query), operationSite);
		return R.data(pages);
	}



	/**
	 * 网点基础信息表 新增
	 */
/*	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationSite")
	public R save(@Valid @RequestBody OperationSiteVO operationSite) {
		return operationSiteService.saveSite(operationSite);
	}*/

	/**
	 * 网点基础信息表 修改
	 */
/*	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationSite")
	public R update(@Valid @RequestBody OperationSiteVO operationSite) {
		return operationSiteService.updateSite(operationSite);
	}*/

	/**
	 * 启用禁用
	 */
	@PostMapping("/updateStatus")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "启用/禁用", description  = "传入参数")
	public R updateStatus(@Parameter(name = "主键id", required = true) @RequestParam String id,@Parameter(name = "状态值", required = true) @RequestParam String status){
		return R.status(operationSiteService.updateStatus(id,status));
	}


	/**
	 * 网点基础信息表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "主键", required = true) @RequestParam String id) {
		return R.status(operationSiteService.deleteSite(id));
	}

	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationSite")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationSite")
	public void exportOperationSite(OperationSiteVO operationSite, BladeUser bladeUser, HttpServletResponse response) {
		List<OperationSiteExcel> list = operationSiteService.selectOperationSiteList(operationSite);

		ExcelUtil.export(response, "网点基础信息表数据" + DateUtil.time(), "网点基础信息表数据表", list, OperationSiteExcel.class);
	}

}
