/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.operation.pojo.dto.OperationSiteEmployeeDTO;
import org.springblade.modules.operation.pojo.vo.EditSiteEmpAuthVo;
import org.springblade.modules.system.pojo.entity.User;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationSiteEmployeeEntity;
import org.springblade.modules.operation.pojo.vo.OperationSiteEmployeeVO;
import org.springblade.modules.operation.excel.OperationSiteEmployeeExcel;
import org.springblade.modules.operation.wrapper.OperationSiteEmployeeWrapper;
import org.springblade.modules.operation.service.IOperationSiteEmployeeService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 服务商员工信息表 控制器
 *
 * @author lzp
 * @since 2024-06-18
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationSiteEmployee")
@Tag(name = "服务商员工信息表", description = "服务商员工信息表接口")
public class OperationSiteEmployeeController extends BladeController {

	private final IOperationSiteEmployeeService operationSiteEmployeeService;

	/**
	 * 服务商员工信息表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationSiteEmployee")
	public R<OperationSiteEmployeeVO> detail(String id) {
		OperationSiteEmployeeVO detail = operationSiteEmployeeService.getSiteEmployee(id);
		return R.data(detail);
	}
	/**
	 * 服务商员工信息表  不分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "不分页", description  = "传入operationSiteEmployee")
	public R<IPage<OperationSiteEmployeeVO>> list(OperationSiteEmployeeVO operationSiteEmploye, Query query) {
		IPage page =Condition.getPage(query);
		page.setSize(100L);
		operationSiteEmploye.setIsDeleted(0);
		IPage<OperationSiteEmployeeVO> pages = operationSiteEmployeeService.selectOperationSiteEmployeePage(page, operationSiteEmploye);
		return R.data(pages);
	}

	/**
	 * 服务商员工信息表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationSiteEmployee")
	public R<IPage<OperationSiteEmployeeVO>> page(OperationSiteEmployeeVO operationSiteEmployee, Query query) {

		IPage<OperationSiteEmployeeVO> pages = operationSiteEmployeeService.selectOperationSiteEmployeePage(Condition.getPage(query), operationSiteEmployee);
		return R.data(pages);
	}

	/**
	 * 服务商员工信息表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationSiteEmployee")
	public R save(@Valid @RequestBody OperationSiteEmployeeDTO operationSiteEmployee) {
		return operationSiteEmployeeService.saveSiteEmp(operationSiteEmployee);
	}

	/**
	 * 服务商员工信息表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationSiteEmployee")
	public R update(@Valid @RequestBody OperationSiteEmployeeDTO operationSiteEmployee) {
		return R.status(operationSiteEmployeeService.updateSiteEmp(operationSiteEmployee));
	}

	/**
	 * 服务商员工信息表 新增或修改
	 */
	@PostMapping("/updateStatus")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "禁用或启用", description  = "传入id")
	public R updateStatus(@Parameter(name = "主键id", required = true) @RequestParam String id,@Parameter(name = "状态值", required = true) @RequestParam String status) {
		return R.status(operationSiteEmployeeService.updateEmpStatus(id, status));
	}

	/**
	 * 服务商员工信息表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "主键", required = true) @RequestParam String id) {
		return R.status(operationSiteEmployeeService.deleteSiteEmp(id));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationSiteEmployee")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationSiteEmployee")
	public void exportOperationSiteEmployee(@Parameter(hidden = true) @RequestParam Map<String, Object> operationSiteEmployee, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationSiteEmployeeEntity> queryWrapper = Condition.getQueryWrapper(operationSiteEmployee, OperationSiteEmployeeEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(OperationSiteEmployee::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(OperationSiteEmployeeEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<OperationSiteEmployeeExcel> list = operationSiteEmployeeService.exportOperationSiteEmployee(queryWrapper);
		ExcelUtil.export(response, "服务商员工信息表数据" + DateUtil.time(), "服务商员工信息表数据表", list, OperationSiteEmployeeExcel.class);
	}

	//编辑服务商员工数据权限
	@PostMapping("/editSiteEmpAuth")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "编辑服务商员工数据权限", description  = "传入siteEmpAuthVo")
	public R editSiteEmpAuth(@Valid @RequestBody EditSiteEmpAuthVo siteEmpAuthVo){
      return R.status(operationSiteEmployeeService.editSiteEmpAuth(siteEmpAuthVo));
	}

}
