/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationSiteRoleEntity;
import org.springblade.modules.operation.pojo.vo.OperationSiteRoleVO;
import org.springblade.modules.operation.excel.OperationSiteRoleExcel;
import org.springblade.modules.operation.wrapper.OperationSiteRoleWrapper;
import org.springblade.modules.operation.service.IOperationSiteRoleService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 服务商员工角色 控制器
 *
 * @author lzp
 * @since 2024-06-19
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationSiteRole")
@Tag(name = "服务商员工角色", description = "服务商员工角色接口")
public class OperationSiteRoleController extends BladeController {

	private final IOperationSiteRoleService operationSiteRoleService;

	/**
	 * 服务商员工角色 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationSiteRole")
	public R<OperationSiteRoleVO> detail(OperationSiteRoleEntity operationSiteRole) {
		OperationSiteRoleEntity detail = operationSiteRoleService.getOne(Condition.getQueryWrapper(operationSiteRole));
		return R.data(OperationSiteRoleWrapper.build().entityVO(detail));
	}
	/**
	 * 服务商员工角色 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "不分页", description  = "传入operationSiteRole")
	public R<List<OperationSiteRoleEntity>> list(BladeUser bladeUser) {
		List<OperationSiteRoleEntity> pages = operationSiteRoleService.roleList(bladeUser);
		return R.data(pages);
	}

	/**
	 * 服务商员工角色 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationSiteRole")
	public R<IPage<OperationSiteRoleVO>> page(OperationSiteRoleVO operationSiteRole, Query query) {
		IPage<OperationSiteRoleVO> pages = operationSiteRoleService.selectOperationSiteRolePage(Condition.getPage(query), operationSiteRole);
		return R.data(pages);
	}

	/**
	 * 服务商员工角色 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationSiteRole")
	public R save(@Valid @RequestBody OperationSiteRoleEntity operationSiteRole) {
		return R.status(operationSiteRoleService.saveRole(operationSiteRole));
	}

	/**
	 * 服务商员工角色 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationSiteRole")
	public R update(@Valid @RequestBody OperationSiteRoleEntity operationSiteRole) {
		return R.status(operationSiteRoleService.updateById(operationSiteRole));
	}

	/**
	 * 服务商员工角色 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入operationSiteRole")
	public R submit(@Valid @RequestBody OperationSiteRoleEntity operationSiteRole) {
		return R.status(operationSiteRoleService.saveOrUpdate(operationSiteRole));
	}

	/**
	 * 服务商员工角色 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String id) {
		return R.status(operationSiteRoleService.deletaStatus(id));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationSiteRole")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationSiteRole")
	public void exportOperationSiteRole(@Parameter(hidden = true) @RequestParam Map<String, Object> operationSiteRole, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationSiteRoleEntity> queryWrapper = Condition.getQueryWrapper(operationSiteRole, OperationSiteRoleEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(OperationSiteRole::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(OperationSiteRoleEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<OperationSiteRoleExcel> list = operationSiteRoleService.exportOperationSiteRole(queryWrapper);
		ExcelUtil.export(response, "服务商员工角色数据" + DateUtil.time(), "服务商员工角色数据表", list, OperationSiteRoleExcel.class);
	}

}
