/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.pojo.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import jakarta.validation.constraints.NotEmpty;
import lombok.Data;
import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * 服务商员工信息表 实体类
 *
 * @author lzp
 * @since 2024-06-18
 */
@Data
@TableName("sf_operation_site_employee")
@Schema(description = "OperationSiteEmployee对象")
public class OperationSiteEmployeeEntity implements Serializable {



	@Schema(
		description = "主键id"
	)
	@TableId(
		value = "id",
		type = IdType.ASSIGN_UUID
	)
	private String id;

	/**
	 * 对应user表记录id
	 */
	@Schema(description = "对应user表记录id")
	private Long userId;
	/**
	 * 姓名
	 */
	@Schema(description = "姓名")
	private String name;
	/**
	 * 手机号码
	 */
	@Schema(description = "手机号码")
	private String mobile;
	/**
	 * 入职时间
	 */
	@JsonFormat(pattern ="yyyy-MM-dd")
	@Schema(description = "入职时间")
	private Date hiredate;
	/**
	 * 1.信息员 2.工程师 3.备件员
	 */
	@Schema(description = "1.信息员 2.工程师 3.备件员")
	@JsonSerialize(
		using = ToStringSerializer.class
	)
	private Long role;
	/**
	 * 服务品类
	 */
	@Schema(description = "服务品类")
	private String category;
	/**
	 * 创建人user_id
	 */
	@Schema(description = "创建人user_id")
	private String createBy;

	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date createTime;
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date updateTime;

	/**
	 * 覆盖省份,多个用逗号隔开
	 */
	@Schema(description = "覆盖省份,多个用逗号隔开")
	private String provinces;

	@Schema(description = "覆盖市，多个用逗号隔开")
	private String city;
	/**
	 * 覆盖区县，多个用逗号隔开
	 */
	@Schema(description = "覆盖区县，多个用逗号隔开")
	private String area;

	/**
	 * 服务商id
	 */
	@Schema(description = "服务商id",requiredMode = Schema.RequiredMode.REQUIRED)
	private String siteId;
	/**
	 * 工作年限
	 */
	@Schema(description = "工作年限")
	private String years;
	/**
	 * 相关资质图片
	 */
	@Schema(description = "相关资质图片")
	private String licenseImg;
	/**
	 * 诚信服务二维码 0.有 1.没有
	 */
	@Schema(description = "诚信服务二维码 0.有 1.没有")
	private String qrcode;

	@Schema(
		description = "是否已删除"
	)
	private Integer isDeleted;



}
