/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.order.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.order.pojo.vo.SelectVo;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.order.pojo.entity.OperationOrderCallbackEntity;
import org.springblade.modules.order.pojo.vo.OperationOrderCallbackVO;
import org.springblade.modules.order.excel.OperationOrderCallbackExcel;
import org.springblade.modules.order.wrapper.OperationOrderCallbackWrapper;
import org.springblade.modules.order.service.IOperationOrderCallbackService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工单回访表 控制器
 *
 * @author BladeX
 * @since 2024-06-21
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationOrderCallback")
@Tag(name = "工单回访表", description = "工单回访表接口")
public class OperationOrderCallbackController extends BladeController {

	private final IOperationOrderCallbackService operationOrderCallbackService;


	/**
	 * 工单回访
	 */
	@PostMapping("/addOrderCllBack")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "工单回访", description  = "传入operationOrderCallback")
	public R addOrderCllBack(@Valid @RequestBody OperationOrderCallbackEntity operationOrderCallback) {
		return R.status(operationOrderCallbackService.addOrderCllBack(operationOrderCallback));
	}

    //获取设备品牌
	@GetMapping("/getAllBrand")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "获取设备品牌")
	public R getAllBrand(){
		List<SelectVo> list=operationOrderCallbackService.getAllBrand();
		return R.data(list);
	}

}
