/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.order.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.order.pojo.entity.OperationOrderDispatchEntity;
import org.springblade.modules.order.pojo.vo.OperationOrderDispatchVO;
import org.springblade.modules.order.wrapper.OperationOrderDispatchWrapper;
import org.springblade.modules.order.service.IOperationOrderDispatchService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工单平台派工表 控制器
 *
 * @author lzp
 * @since 2024-06-20
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationOrderDispatch")
@Tag(name = "工单平台派工表", description = "工单平台派工表接口")
public class OperationOrderDispatchController extends BladeController {

	private final IOperationOrderDispatchService operationOrderDispatchService;


	/**
	 * 工单平台派工表 新增
	 */
	@PostMapping("/dispatchOrderSite")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "派工服务商", description  = "传入operationOrderDispatch")
	public R dispatchOrderSite(@Parameter(name = "工单id", required = true) @RequestParam String orderId,
							   @Parameter(name = "服务商id", required = true) @RequestParam String siteId) {
		return R.status(operationOrderDispatchService.dispatchOrderSite(orderId, siteId));
	}


	@PostMapping(value = "/receiveOrder")
	@ApiOperationSupport(order = 10)
	@Operation(summary = "服务商接单", description = "传入orderIds")
	public R receiveOrder(@Parameter(name = "orderIds", description = "工单id,多个逗号拼接", required = true) @RequestParam String orderIds) {
		return R.status(operationOrderDispatchService.receiveOrder(orderIds));
	}

}
