/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.order.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.order.excel.OperationOrderFeedbackExcel;
import org.springblade.modules.order.pojo.entity.OperationOrderFeedbackEntity;
import org.springblade.modules.order.pojo.vo.OperationOrderFeedbackVO;
import org.springblade.modules.order.service.IOperationOrderFeedbackService;
import org.springblade.modules.order.wrapper.OperationOrderFeedbackWrapper;
import org.springframework.web.bind.annotation.*;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 工单反馈表 控制器
 *
 * @author lzp
 * @since 2024-06-21
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationOrderFeedback")
@Tag(name = "工单反馈表", description = "工单反馈表接口")
public class OperationOrderFeedbackController extends BladeController {

	private final IOperationOrderFeedbackService operationOrderFeedbackService;

	/**
	 * 工单过程反馈 + 服务完成
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "工单过程反馈 + 服务完成", description  = "传入operationOrderFeedback")
	public R save(@Valid @RequestBody OperationOrderFeedbackVO operationOrderFeedback) {
		return R.status(operationOrderFeedbackService.feedbackSealingSheet(operationOrderFeedback));
	}

	//电站的过程反馈信息
    @GetMapping(value = "/listFeedback")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "过程反馈信息")
	public R listFeedback(@Parameter(name = "orderId",description = "工单id",required = true) @RequestParam String orderId){
		List<OperationOrderFeedbackEntity> list=operationOrderFeedbackService.listFeedback(orderId);
		return R.data(list);
	}

}
