/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.order.excel;


import lombok.Data;

import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 工单反馈表 Excel实体类
 *
 * @author lzp
 * @since 2024-06-21
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class OperationOrderFeedbackExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * order表主键id
	 */
	@ColumnWidth(20)
	@ExcelProperty("order表主键id")
	private String orderId;
	/**
	 * 反馈时间
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈时间")
	private Date feedbackTime;
	/**
	 * 反馈人id:对应角色子表主键id
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈人id:对应角色子表主键id")
	private String feedbackId;
	/**
	 * 反馈人姓名
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈人姓名")
	private String feedbackName;
	/**
	 * 反馈文字内容,维修工反馈维修过程内容，信息员可反馈记录客户催单/（缺件：+配件描述）
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈文字内容,维修工反馈维修过程内容，信息员可反馈记录客户催单/（缺件：+配件描述）")
	private String feedback;
	/**
	 * 反馈结果
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈结果")
	private String feedbackResult;
	/**
	 * 反馈设备主图片路径（可存放多个图片路径，以","隔开）
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈设备主图片路径（可存放多个图片路径，以隔开）")
	private String feedbackImg;
	/**
	 * 备注
	 */
	@ColumnWidth(20)
	@ExcelProperty("备注")
	private String remarks;
	/**
	 * 服务商id
	 */
	@ColumnWidth(20)
	@ExcelProperty("服务商id")
	private String siteId;
	/**
	 * 反馈类型：0 过程反馈 1服务完成反馈 2服务无法完成反馈
	 */
	@ColumnWidth(20)
	@ExcelProperty("反馈类型：0 过程反馈 1服务完成反馈 2服务无法完成反馈")
	private String feedbackType;
	/**
	 * 设备名称 （安装时会使用）
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备名称 （安装时会使用）")
	private String deviceName;
	/**
	 * 设备规格型号
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备规格型号")
	private String deviceModel;
	/**
	 * 设备机身编码
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备机身编码")
	private String deviceMachineCode;
	/**
	 * 钢结构图片
	 */
	@ColumnWidth(20)
	@ExcelProperty("钢结构图片")
	private String steelStructureImg;
	/**
	 * 支架图片
	 */
	@ColumnWidth(20)
	@ExcelProperty("支架图片")
	private String bracketImg;
	/**
	 * 组件图片
	 */
	@ColumnWidth(20)
	@ExcelProperty("组件图片")
	private String assemblyImg;
	/**
	 * 设备接线
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备接线")
	private String deviceImg;
	/**
	 * 是否遮挡照片
	 */
	@ColumnWidth(20)
	@ExcelProperty("是否遮挡照片")
	private String shelterImg;
	/**
	 * 全景照片
	 */
	@ColumnWidth(20)
	@ExcelProperty("全景照片")
	private String panoramaImg;

}
