/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.partner.excel;


import lombok.Data;

import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 合作商基础信息表 Excel实体类
 *
 * @author lzp
 * @since 2024-06-18
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class PartnerExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * 授权类型:1.接口授权 2.账号授权 3.无授权
	 */
	@ColumnWidth(20)
	@ExcelProperty("授权类型:1.接口授权 2.账号授权 3.无授权")
	private String authType;
	/**
	 * 对应user表记录id
	 */
	@ColumnWidth(20)
	@ExcelProperty("对应user表记录id")
	private String userId;
	/**
	 * 资方名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("资方名称")
	private String name;
	/**
	 * 联系电话
	 */
	@ColumnWidth(20)
	@ExcelProperty("联系电话")
	private String mobile;
	/**
	 * 负责人
	 */
	@ColumnWidth(20)
	@ExcelProperty("负责人")
	private String personInCharge;
	/**
	 * 省
	 */
	@ColumnWidth(20)
	@ExcelProperty("省")
	private String province;
	/**
	 * 市
	 */
	@ColumnWidth(20)
	@ExcelProperty("市")
	private String city;
	/**
	 * 区县
	 */
	@ColumnWidth(20)
	@ExcelProperty("区县")
	private String area;
	/**
	 * 详细地址
	 */
	@ColumnWidth(20)
	@ExcelProperty("详细地址")
	private String address;
	/**
	 * 合作商logo
	 */
	@ColumnWidth(20)
	@ExcelProperty("合作商logo")
	private String logo;
	/**
	 * 授权项目
	 */
	@ColumnWidth(20)
	@ExcelProperty("授权项目")
	private String authorizedId;
	/**
	 * 创建人user_id
	 */
	@ColumnWidth(20)
	@ExcelProperty("创建人user_id")
	private String createBy;
	/**
	 * 创建人
	 */
	@ColumnWidth(20)
	@ExcelProperty("创建人")
	private String creator;
	/**
	 * 更新人user_id
	 */
	@ColumnWidth(20)
	@ExcelProperty("更新人user_id")
	private String updateBy;
	/**
	 * 更新人
	 */
	@ColumnWidth(20)
	@ExcelProperty("更新人")
	private String updateName;
	/**
	 * 营业执照
	 */
	@ColumnWidth(20)
	@ExcelProperty("营业执照")
	private String certificationImg;
	/**
	 * 统一信用代码
	 */
	@ColumnWidth(20)
	@ExcelProperty("统一信用代码")
	private String unifiedCreditCode;
	/**
	 * 营业执照文件名（包含后缀）。
	 */
	@ColumnWidth(20)
	@ExcelProperty("营业执照文件名（包含后缀）。")
	private String certificationImgName;
	/**
	 * 是否开启超龄：1.开启 2.关闭
	 */
	@ColumnWidth(20)
	@ExcelProperty("是否开启超龄：1.开启 2.关闭")
	private String overageFlag;
	/**
	 * 是否开通二类卡：1.是 0.否
	 */
	@ColumnWidth(20)
	@ExcelProperty("是否开通二类卡：1.是 0.否")
	private String isOpenBankCard;

}
